@@ -9,9 +9,10 @@
 #define QFCITXPLATFORMINPUTCONTEXT_H
 
 #include "fcitxcandidatewindow.h"
-#include "fcitxqtinputcontextproxy.h"
 #include "fcitxqtwatcher.h"
+#include "hybridinputcontext.h"
 #include <QDBusConnection>
+#include <QDBusPendingCallWatcher>
 #include <QDBusServiceWatcher>
 #include <QGuiApplication>
 #include <QKeyEvent>
@@ -32,7 +33,7 @@ class FcitxQtICData : public QObject {
 public:
     FcitxQtICData(QFcitxPlatformInputContext *context, QWindow *window);
     FcitxQtICData(const FcitxQtICData &that) = delete;
-    ~FcitxQtICData();
+    ~FcitxQtICData() override;
 
     FcitxCandidateWindow *candidateWindow();
 
@@ -41,7 +42,7 @@ public:
     void resetCandidateWindow();
 
     quint64 capability = 0;
-    FcitxQtInputContextProxy *proxy;
+    HybridInputContext *proxy;
     QRect rect;
     // Last key event forwarded.
     std::unique_ptr<QKeyEvent> event;
@@ -61,7 +62,7 @@ class ProcessKeyWatcher : public QDBusPendingCallWatcher {
     Q_OBJECT
 public:
     ProcessKeyWatcher(const QKeyEvent &event, QWindow *window,
-                      const QDBusPendingCall &call, QObject *parent = 0)
+                      const QDBusPendingCall &call, QObject *parent = nullptr)
         : QDBusPendingCallWatcher(call, parent),
           event_(event.type(), event.key(), event.modifiers(),
                  event.nativeScanCode(), event.nativeVirtualKey(),
@@ -69,8 +70,6 @@ public:
                  event.count()),
           window_(window) {}
 
-    virtual ~ProcessKeyWatcher() {}
-
     const QKeyEvent &keyEvent() { return event_; }
 
     QWindow *window() { return window_.data(); }
@@ -82,22 +81,25 @@ private:
 
 struct XkbContextDeleter {
     static inline void cleanup(struct xkb_context *pointer) {
-        if (pointer)
+        if (pointer) {
             xkb_context_unref(pointer);
+        }
     }
 };
 
 struct XkbComposeTableDeleter {
     static inline void cleanup(struct xkb_compose_table *pointer) {
-        if (pointer)
+        if (pointer) {
             xkb_compose_table_unref(pointer);
+        }
     }
 };
 
 struct XkbComposeStateDeleter {
     static inline void cleanup(struct xkb_compose_state *pointer) {
-        if (pointer)
+        if (pointer) {
             xkb_compose_state_unref(pointer);
+        }
     }
 };
 
@@ -105,19 +107,24 @@ class QFcitxPlatformInputContext : public QPlatformInputContext {
     Q_OBJECT
 public:
     QFcitxPlatformInputContext();
-    virtual ~QFcitxPlatformInputContext();
+    ~QFcitxPlatformInputContext() override;
 
     bool isValid() const override;
     void setFocusObject(QObject *object) override;
-    void invokeAction(QInputMethod::Action, int cursorPosition) override;
+    void invokeAction(QInputMethod::Action imAction,
+                      int cursorPosition) override;
     void reset() override;
     void commit() override;
-    void update(Qt::InputMethodQueries quries) override;
+    void update(Qt::InputMethodQueries queries) override;
     bool filterEvent(const QEvent *event) override;
     QLocale locale() const override;
     bool hasCapability(Capability capability) const override;
+    void showInputPanel() override;
+    void hideInputPanel() override;
+    bool isInputPanelVisible() const override;
 
-    FcitxQtWatcher *watcher() { return watcher_; }
+    auto *watcher() { return watcher_; }
+    auto *fcitx4Watcher() { return fcitx4Watcher_; }
 
     // Use Wrapper as suffix to avoid upstream add function with same name.
     QObject *focusObjectWrapper() const;
@@ -154,9 +161,9 @@ private Q_SLOTS:
 
 private:
     bool processCompose(unsigned int keyval, unsigned int state,
-                        bool isRelaese);
+                        bool isRelease);
     QKeyEvent *createKeyEvent(unsigned int keyval, unsigned int state,
-                              bool isRelaese, const QKeyEvent *event);
+                              bool isRelease, const QKeyEvent *event);
     void forwardEvent(QWindow *window, const QKeyEvent &event);
 
     void addCapability(FcitxQtICData &data, quint64 capability,
@@ -179,15 +186,19 @@ private:
 
     void updateCapability(const FcitxQtICData &data);
     void createICData(QWindow *w);
-    FcitxQtInputContextProxy *validIC();
-    FcitxQtInputContextProxy *validICByWindow(QWindow *window);
+    HybridInputContext *validIC() const;
+    HybridInputContext *validICByWindow(QWindow *window) const;
     bool filterEventFallback(unsigned int keyval, unsigned int keycode,
-                             unsigned int state, bool isRelaese);
+                             unsigned int state, bool isRelease);
 
     void updateCursorRect();
     bool objectAcceptsInputMethod() const;
+    bool shouldDisableInputMethod() const;
+
+    void updateInputPanelVisible();
 
     FcitxQtWatcher *watcher_;
+    Fcitx4Watcher *fcitx4Watcher_;
     QString preedit_;
     QString commitPreedit_;
     FcitxQtFormattedPreeditList preeditList_;
@@ -198,6 +209,7 @@ private:
     QPointer<QWindow> lastWindow_;
     QPointer<QObject> lastObject_;
     bool destroy_;
+    bool virtualKeyboardVisible_;
     QScopedPointer<struct xkb_context, XkbContextDeleter> xkbContext_;
     QScopedPointer<struct xkb_compose_table, XkbComposeTableDeleter>
         xkbComposeTable_;
@@ -205,6 +217,7 @@ private:
         xkbComposeState_;
     QLocale locale_;
     FcitxTheme *theme_ = nullptr;
+    bool inputPanelVisible_ = false;
 };
 } // namespace fcitx
 
