@@ -167,8 +167,8 @@ void QuickPhraseModel::load(const QString &file, bool append) {
     } else
         setNeedSave(true);
     futureWatcher_ = new QFutureWatcher<QStringPairList>(this);
-    futureWatcher_->setFuture(QtConcurrent::run<QStringPairList>(
-        this, &QuickPhraseModel::parse, file));
+    futureWatcher_->setFuture(
+        QtConcurrent::run([this, file]() { return parse(file); }));
     connect(futureWatcher_, &QFutureWatcherBase::finished, this,
             &QuickPhraseModel::loadFinished);
 }
@@ -216,17 +216,17 @@ void QuickPhraseModel::loadFinished() {
 }
 
 QFutureWatcher<bool> *QuickPhraseModel::save(const QString &file) {
-    QFutureWatcher<bool> *futureWatcher = new QFutureWatcher<bool>(this);
-    futureWatcher->setFuture(QtConcurrent::run<bool>(
-        this, &QuickPhraseModel::saveData, file, list_));
+    auto *futureWatcher = new QFutureWatcher<bool>(this);
+    futureWatcher->setFuture(QtConcurrent::run(
+        [this, file, list = list_]() { return saveData(file, list); }));
     connect(futureWatcher, &QFutureWatcherBase::finished, this,
             &QuickPhraseModel::saveFinished);
     return futureWatcher;
 }
 
-void QuickPhraseModel::saveData(QTextStream &dev) {
-    for (int i = 0; i < list_.size(); i++) {
-        dev << list_[i].first << "\t" << escapeValue(list_[i].second) << "\n";
+void QuickPhraseModel::saveDataToStream(QTextStream &dev) {
+    for (const auto &item : list_) {
+        dev << item.first << "\t" << escapeValue(item.second) << "\n";
     }
 }
 
@@ -263,10 +263,10 @@ bool QuickPhraseModel::saveData(const QString &file,
             if (!tempFile.open(fd, QIODevice::WriteOnly)) {
                 return false;
             }
-            for (int i = 0; i < list.size(); i++) {
-                tempFile.write(list[i].first.toUtf8());
+            for (const auto &item : list) {
+                tempFile.write(item.first.toUtf8());
                 tempFile.write("\t");
-                tempFile.write(escapeValue(list[i].second).toUtf8());
+                tempFile.write(escapeValue(item.second).toUtf8());
                 tempFile.write("\n");
             }
             tempFile.close();
