@@ -47,6 +47,11 @@ QDBusPendingReply<> FcitxQtInputContextProxy::focusOut() {
     return d->icproxy_->FocusOut();
 }
 
+QDBusPendingReply<> FcitxQtInputContextProxy::hideVirtualKeyboard() {
+    Q_D(FcitxQtInputContextProxy);
+    return d->icproxy_->HideVirtualKeyboard();
+}
+
 QDBusPendingReply<bool> FcitxQtInputContextProxy::processKeyEvent(
     unsigned int keyval, unsigned int keycode, unsigned int state, bool type,
     unsigned int time) {
@@ -59,6 +64,12 @@ QDBusPendingReply<> FcitxQtInputContextProxy::reset() {
     return d->icproxy_->Reset();
 }
 
+QDBusPendingReply<>
+FcitxQtInputContextProxy::setSupportedCapability(qulonglong caps) {
+    Q_D(FcitxQtInputContextProxy);
+    return d->icproxy_->SetSupportedCapability(caps);
+}
+
 QDBusPendingReply<> FcitxQtInputContextProxy::setCapability(qulonglong caps) {
     Q_D(FcitxQtInputContextProxy);
     return d->icproxy_->SetCapability(caps);
@@ -90,6 +101,11 @@ FcitxQtInputContextProxy::setSurroundingTextPosition(unsigned int cursor,
     return d->icproxy_->SetSurroundingTextPosition(cursor, anchor);
 }
 
+QDBusPendingReply<> FcitxQtInputContextProxy::showVirtualKeyboard() {
+    Q_D(FcitxQtInputContextProxy);
+    return d->icproxy_->ShowVirtualKeyboard();
+}
+
 QDBusPendingReply<> FcitxQtInputContextProxy::prevPage() {
     Q_D(FcitxQtInputContextProxy);
     return d->icproxy_->PrevPage();
@@ -111,6 +127,11 @@ QDBusPendingReply<> FcitxQtInputContextProxy::invokeAction(unsigned int action,
     return d->icproxy_->InvokeAction(action, cursor);
 }
 
+bool FcitxQtInputContextProxy::isVirtualKeyboardVisible() {
+    Q_D(FcitxQtInputContextProxy);
+    return d->isVirtualKeyboardVisible_;
+}
+
 bool FcitxQtInputContextProxy::supportInvokeAction() const {
     Q_D(const FcitxQtInputContextProxy);
     return d->supportInvokeAction_;
